
* Ten-fold cross validation

$eolcom !

$offsymlist offsymxref offuellist offuelxref
option limrow = 0;
option limcol = 0;
option solprint = off;

$setglobal dataset "galaxy"

$include "%dataset%.inc"

set a /1*%ele_1%/;	! set for category 1
set b /1*%ele_2%/;	! set for category 2
set o /1*%observ%/;		! observations

set p /1*10/;	! folds to perform
set f /1*10/;	! maximum features to select
set l /1*20/;   ! number of lots to perform

scalar max_l / 20 /;
scalar max_f / 8 /;

alias (a, a_temp), (b, b_temp), (p, p_temp);

! Read in the data from the data files
parameter a_data(a, o) /
$offlisting
$include "%dataset%_a_data.inc"
$onlisting
/;

parameter b_data(b, o) /
$offlisting
$include "%dataset%_b_data.inc"
$onlisting
/;

parameter a_perm(l, a) /
$offlisting
$include "%dataset%_a_perm.inc"
$onlisting
/;

parameter b_perm(l, b) /
$offlisting
$include "%dataset%_b_perm.inc"
$onlisting
/;

! Generate the permutations for the ten folds
scalar a_num, b_num;
parameter a_permi(a, p, l), b_permi(b, p, l);

a_num = floor(card(a) / card(p));
b_num = floor(card(b) / card(p));

a_permi(a, '1', l) = a_perm(l, a);
b_permi(b, '1', l) = b_perm(l, b);

loop(p_temp$(ord(p_temp) lt card(p)),
          a_permi(a, p, l)$(ord(p) = ord(p_temp)+1) = a_permi(a++a_num, p-1, l);
          b_permi(b, p, l)$(ord(p) = ord(p_temp)+1) = b_permi(b++b_num, p-1, l);
    );

! Set up the problem using dynamic sets
set a_test(a);
set a_trai(a);
set b_test(b);
set b_trai(b);

scalar w_tol /1/;
scalar num_f;

positive variables a_class(a), b_class(b);
variables c, weight(o), gamma;
binary variable y(o);

equations c_def, a_def(a), b_def(b), w_def1(o), w_def2(o), y_def;

c_def..
	 c =e= (sum(a_trai, a_class(a_trai))/card(a_trai) +
                sum(b_trai, b_class(b_trai))/card(b_trai));

a_def(a_trai)..
	-sum(o, a_data(a_trai, o)*weight(o)) + gamma + 1 =l= a_class(a_trai);

b_def(b_trai)..
	 sum(o, b_data(b_trai, o)*weight(o)) - gamma + 1 =l= b_class(b_trai);

w_def1(o)..
	 weight(o) =l= w_tol*y(o);

w_def2(o)..
	 weight(o) =g= -w_tol*y(o);

y_def..
	 sum(o, y(o)) =e= num_f;

model train /all/;
train.optfile = 1;
train.optca = 0;
train.optcr = 0;
train.reslim = 20000;
train.iterlim = 100000000;
train.workspace = 20;

! Solve all of the problems and generate a final report
parameter obj(f, p, l);
parameter sol(f, p, l, o);
parameter report(f, p, l);
parameter comp(f, l);
parameter comb(f);

train.cutoff = 100;

obj(f, p, l) = 0;
sol(f, p, l, o) = 0;

num_f = 1;

option mip = xprcon;
loop(l$(ord(l) le max_l),

     loop(p,

          a_test(a) = no;
          a_trai(a) = no;
          loop(a$(ord(a) <= a_num),

               a_test(a_temp)$(ord(a_temp) eq a_permi(a, p, l)) = yes;
    
              );

          loop(a$(ord(a) > a_num),

               a_trai(a_temp)$(ord(a_temp) eq a_permi(a, p, l)) = yes;

              );

          b_test(b) = no;
          b_trai(b) = no;
          loop(b$(ord(b) <= b_num),

               b_test(b_temp)$(ord(b_temp) eq b_permi(b, p, l)) = yes;

              );

          loop(b$(ord(b) > b_num),

               b_trai(b_temp)$(ord(b_temp) eq b_permi(b, p, l)) = yes;

              );
                    
          solve train using mip minimizing c;
     );
);

loop(f$(ord(f) le max_f),

     loop(l$(ord(l) le max_l),

          loop(p,

               num_f = ord(f);

               a_test(a) = no;
               a_trai(a) = no;
               loop(a$(ord(a) <= a_num),

                    a_test(a_temp)$(ord(a_temp) eq a_permi(a, p, l)) = yes;
    
                   );

               loop(a$(ord(a) > a_num),

                    a_trai(a_temp)$(ord(a_temp) eq a_permi(a, p, l)) = yes;

                   );

               b_test(b) = no;
               b_trai(b) = no;
               loop(b$(ord(b) <= b_num),

                    b_test(b_temp)$(ord(b_temp) eq b_permi(b, p, l)) = yes;

                   );

               loop(b$(ord(b) > b_num),

                    b_trai(b_temp)$(ord(b_temp) eq b_permi(b, p, l)) = yes;

                   );
                    
	       option mip = rescon;
               solve train using mip minimizing c;

	       if (execerror,

		   execerror = 0;

		   if (ord(f) eq 1,
		       train.cutoff = 100;
                   else
	               train.cutoff = obj(f - 1, p, l);
                       );
	           
		   option mip = xpress;
		   solve train using mip minimizing c;

                  );

               if (train.solvestat eq 7 or train.modelstat eq 11,

		   if (ord(f) eq 1,
		       train.cutoff = 100;
                   else
	               train.cutoff = obj(f - 1, p, l);
                       );
	           
		   option mip = xpress;
		   solve train using mip minimizing c;

                  );

               obj(f, p, l) = c.l;
	       sol(f, p, l, o) = y.l(o);

	       report(f, p, l) = 
                    (sum(a_test, 
	              1$( -sum(o, a_data(a_test, o)*weight.l(o)) + gamma.l ge 0)
                        ) +
		     sum(b_test,
	              1$( sum(o,  b_data(b_test, o)*weight.l(o)) - gamma.l ge 0)
                        )
                    ) / (card(a_test) + card(b_test));

	       if (ord(f) lt max_f,
              
		   num_f = ord(f) + 1;
	           train.cutoff = obj(f, p, l);

	           option mip = xprcon;
                   solve train using mip minimizing c;

                  );
             );
             comp(f, l) = sum(p, report(f, p, l)) / card(p);
         );
    );

comb(f) = sum(l, comp(f, l)) / max_l;

display sol;
display comp;
display comb;
display max_f;
display max_l;

